package com.iohao.mmo.equip.action;

import com.iohao.game.action.skeleton.annotation.ActionController;
import com.iohao.game.action.skeleton.annotation.ActionMethod;
import com.iohao.game.action.skeleton.core.flow.FlowContext;
import com.iohao.game.action.skeleton.protocol.wrapper.StringValue;
import com.iohao.mmo.common.config.GameCode;
import com.iohao.mmo.equip.cmd.EquipCmd;
import com.iohao.mmo.equip.entity.Equip;
import com.iohao.mmo.equip.mapper.EquipMapper;
import com.iohao.mmo.equip.proto.EquipMessage;
import com.iohao.mmo.equip.proto.EquipResetMessage;
import com.iohao.mmo.equip.service.EquipService;
import jakarta.annotation.Resource;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.stereotype.Component;

import java.math.BigDecimal;
import java.util.List;

/**
 * @author 唐斌
 * @date 2023-07-30
 * @description: 装备属性类
 */
@Slf4j
@Component
@ActionController(EquipCmd.cmd)
public class EquipAction {

    @Resource
    EquipService equipService;


    /**
     * 获取某人装备列表信息
     *
     * @param flowContext flowContext
     * @param store 所属位置：1包裹，2仓库，3穿戴
     */
    @ActionMethod(EquipCmd.getEquipList)
    public List<EquipMessage> getEquipList(FlowContext flowContext,int store) { //参数值
        long userId = flowContext.getUserId();
        // 初始化装备数据，暂时放这
        List<Equip> equipList = equipService.listByUser(userId,store);

        return EquipMapper.ME.convert(equipList);
    }

    /**
     * 获取装备信息
     *
     * @param flowContext flowContext
     * @param stringValue itemId
     */
    @ActionMethod(EquipCmd.getEquip)
    public EquipMessage getEquip(FlowContext flowContext, StringValue stringValue) { //参数值
        // 初始化装备数据，暂时放这
        Equip equip = equipService.findById(stringValue.value);
        return EquipMapper.ME.convert(equip);
    }

    /**
     * 分配装备属性点
     *
     * @param flowContext flowContext
     */
    @ActionMethod(EquipCmd.allotEquip)
    public void allotEquip(FlowContext flowContext, EquipMessage equipMessage) { //参数值
        equipService.allotEquip(EquipMapper.ME.convert(equipMessage));
    }

    /**
     * 重新随机总属性点
     *
     * @param flowContext flowContext
     * @param equipResetMessage 重新随机装备总属性点参数对象
     */
    @ActionMethod(EquipCmd.resetEquip)
    public EquipMessage resetEquip(FlowContext flowContext, EquipResetMessage equipResetMessage) { //参数值
        GameCode.objNotFound.assertTrue(StringUtils.isNotBlank(equipResetMessage.itemId)&&StringUtils.isNotBlank(equipResetMessage.excellentRateString));
        Equip equip = equipService.resetEquip((equipResetMessage.itemId), new BigDecimal(equipResetMessage.excellentRateString));
        return EquipMapper.ME.convert(equip);
    }

    /**
     * 批量删除装备
     *
     * @param flowContext flowContext
     */
    @ActionMethod(EquipCmd.delBatch)
    public void delBatch(FlowContext flowContext, StringValue stringValue) { //参数值
        equipService.delBatch(stringValue.value);
    }
}
