/*
 * ioGame
 * Copyright (C) 2021 - 2023  渔民小镇 （262610965@qq.com、luoyizhu@gmail.com） . All Rights Reserved.
 * # iohao.com . 渔民小镇
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */
package com.iohao.mmo.equip.client;

import com.iohao.game.action.skeleton.protocol.wrapper.ByteValueList;
import com.iohao.game.action.skeleton.protocol.wrapper.IntValue;
import com.iohao.game.action.skeleton.protocol.wrapper.StringValue;
import com.iohao.game.external.client.AbstractInputCommandRegion;
import com.iohao.game.external.client.command.InputRequestData;
import com.iohao.game.external.client.kit.ScannerKit;
import com.iohao.game.external.client.kit.SplitParam;
import com.iohao.mmo.common.provide.kit.JsonKit;
import com.iohao.mmo.equip.cmd.EquipCmd;
import com.iohao.mmo.equip.proto.*;
import lombok.extern.slf4j.Slf4j;
import java.util.List;

/**
 * @author 唐斌
 * @date 2023-07-30
 */
@Slf4j
public class EquipInputCommandRegion extends AbstractInputCommandRegion {
    @Override
    public void initInputCommand() {
        this.inputCommandCreate.cmd = EquipCmd.cmd;
        this.inputCommandCreate.cmdName = "装备模块";

        request();

        listen();
    }

    private void listen() {

    }

    private void request() {
        InputRequestData inputRequestData;

        // 动态请求参数（参数值）
        inputRequestData = () -> {
            ScannerKit.log(() -> log.info("请输入获取的装备存储位置：0所有，1包裹，2仓库，3穿戴"));
            int inputValue = ScannerKit.nextInt(0);

            // 请求参数
            IntValue intValue = IntValue.of(inputValue);
            return intValue;
        };
        // 10-1
        ofCommand(EquipCmd.getEquipList).callback(ByteValueList.class, result -> {
            List<EquipMessage> value = result.toList(EquipMessage.class);
            log.info("装备列表信息 : {}", JsonKit.toJsonString(value));
        }).setDescription("获取装备列表信息").setInputRequestData(inputRequestData);

        // 动态请求参数（参数值）
        inputRequestData = () -> {
            ScannerKit.log(() -> log.info("请输入要查询的装备itemId"));
            String id = ScannerKit.nextLine();

            // 请求参数
            StringValue stringValue = StringValue.of(id);

            return stringValue;
        };

        // 10-2
        ofCommand(EquipCmd.getEquip).callback(EquipMessage.class, result -> {
            EquipMessage value = result.getValue();
            log.info("装备信息 : {}", value);
        }).setDescription("查询装备信息").setInputRequestData(inputRequestData);

        // 动态请求参数（参数值）
        inputRequestData = () -> {
            ScannerKit.log(() -> log.info("请输入装备itemId"));
            String id = ScannerKit.nextLine();
            ScannerKit.log(() -> log.info("请输入体质、魔力、力量、耐力、敏捷，格式[体质-魔力-力量-耐力-敏捷]"));
            String inputValue = ScannerKit.nextLine("1-1-1-1-1");
            SplitParam param = new SplitParam(inputValue);
            int constitution = param.getInt(0,0);
            int magicPower = param.getInt(1,0);
            int power = param.getInt(2,0);
            int endurance = param.getInt(3,0);
            int agile = param.getInt(4,0);

            BasicEquipPropertyMessage basicEquipPropertyMessage = new BasicEquipPropertyMessage(constitution,magicPower,power,endurance,agile);
            EquipMessage equipMessage = new EquipMessage();
            equipMessage.basicEquipProperty = basicEquipPropertyMessage;
            equipMessage.id = id;
            return equipMessage;
        };

        // 10-3
        ofCommand(EquipCmd.allotEquip).callback(EquipMessage.class, result -> {
            EquipMessage value = result.getValue();
            log.info("装备属性信息 : {}", value);
        }).setDescription("分配装备属性信息").setInputRequestData(inputRequestData);


        // 动态请求参数（参数值）
        inputRequestData = () -> {
            ScannerKit.log(() -> log.info("请输入装备id和消耗道具id,格式 [装备id-增益百分比]"));
            String inputValue = ScannerKit.nextLine("1-0.3");
            SplitParam param = new SplitParam(inputValue);
            String itemId = param.getString(0);
            String excellentRateString = param.getString(1);

            EquipResetMessage equipResetMessage = new EquipResetMessage();
            equipResetMessage.itemId = itemId;
            equipResetMessage.excellentRateString = excellentRateString;
            return equipResetMessage;
        };

        // 10-4
        ofCommand(EquipCmd.resetEquip).callback(EquipMessage.class, result -> {
            EquipMessage value = result.getValue();
            log.info("装备属性信息 : {}", value);
        }).setDescription("重新随机总属性点").setInputRequestData(inputRequestData);


        // 动态请求参数（参数值）
        inputRequestData = () -> {
            ScannerKit.log(() -> log.info("请输入要删除的装备id,格式 [装备1id-装备2id]"));
            String ids = ScannerKit.nextLine("1-1");

            // 请求参数
            StringValue stringValue = StringValue.of(ids);
            return stringValue;
        };

        // 10-5
        ofCommand(EquipCmd.delBatch).callback(EquipMessage.class, result -> {
            EquipMessage value = result.getValue();
            log.info("装备属性信息 : {}", value);
        }).setDescription("批量删除装备").setInputRequestData(inputRequestData);

        // 10-10
        ofCommand(EquipCmd.initEquipTemplate).callback(EquipTemplateMessage.class, result -> {
            EquipTemplateMessage value = result.getValue();
            log.info("装备库信息 : {}", value);
        }).setDescription("初始化装备库");

        // 动态请求参数（参数值）
        inputRequestData = () -> {
            ScannerKit.log(() -> log.info("请输入装备库id列表,格式 [装备库1id-装备库2id-装备库3id...]"));
            String ids = ScannerKit.nextLine("1-1");

            // 请求参数
            StringValue stringValue = StringValue.of(ids);

            return stringValue;
        };
        // 10-13
        ofCommand(EquipCmd.randomEquip).callback(EquipMessage.class, result -> {
            EquipMessage value = result.getValue();
            log.info("装备信息 : {}", value);
        }).setDescription("根据装备库列表各随机出一件新的装备").setInputRequestData(inputRequestData);
    }
}
